@@ -18,7 +18,6 @@ class Lexer(object):
         ',': 'comma',
         ':': 'colon',
         '@': 'current',
-        '$': 'root',
         '(': 'lparen',
         ')': 'rparen',
         '{': 'lbrace',
@@ -115,22 +114,15 @@ class Lexer(object):
             elif self._current == '!':
                 yield self._match_or_else('=', 'ne', 'not')
             elif self._current == '=':
-                if self._next() == '=':
-                    yield {'type': 'eq', 'value': '==',
-                        'start': self._position - 1, 'end': self._position}
-                    self._next()
+                yield self._match_or_else('=', 'eq', 'assign')
+            elif self._current == '$':
+                if self._peek_may_be_valid_unquoted_identifier():
+                    yield self._consume_variable()
                 else:
-                    if self._current is None:
-                        # If we're at the EOF, we never advanced
-                        # the position so we don't need to rewind
-                        # it back one location.
-                        position = self._position
-                    else:
-                        position = self._position - 1
-                    raise LexerError(
-                        lexer_position=position,
-                        lexer_value='=',
-                        message="Unknown token '='")
+                    yield {'type': 'root',
+                           'value': self._current,
+                           'start': self._position, 'end': self._position + 1}
+                    self._next()
             else:
                 raise LexerError(lexer_position=self._position,
                                  lexer_value=self._current,
@@ -145,6 +137,28 @@ class Lexer(object):
             buff += self._current
         return buff
 
+    def _consume_variable(self):
+        start = self._position
+        buff = self._current
+        self._next()
+        if self._current not in self.START_IDENTIFIER:
+            raise LexerError(
+                lexer_position=start,
+                lexer_value=self._current,
+                message='Invalid variable starting character %s' % self._current)
+        buff += self._current
+        while self._next() in self.VALID_IDENTIFIER:
+            buff += self._current
+        return {'type': 'variable', 'value': buff,
+                'start': start, 'end': start + len(buff)}
+
+    def _peek_may_be_valid_unquoted_identifier(self):
+        if (self._position == self._length - 1):
+            return False
+        else:
+            next = self._chars[self._position + 1]
+            return next in self.START_IDENTIFIER
+
     def _peek_is_next_digit(self):
         if (self._position == self._length - 1):
             return False
